<?
   if(!isset($GLOBALS[__FILE__]))
   {
      $GLOBALS[__FILE__] = true;

      error_reporting(-1);

      define("DATA_DIR",              "../data");
      define("DATA_MOVIE_TITLES",     "movie_titles.txt");
      define("DATA_MOVIE_TITLES_PATH", DATA_DIR . "/" . DATA_MOVIE_TITLES);
      define("DATA_TRAINING_SET",      "training_set");
      define("DATA_TRAINING_SET_PATH", DATA_DIR . "/" . DATA_TRAINING_SET);

      class MovieEntry
      {
         var $mID    = 0;
         var $mYear  = 0;
         var $mTitle = "";

         function __construct($id, $year, $title)
         {
            $this->mID    = $id;
            $this->mYear  = $year;
            $this->mTitle = $title;
         }

         function toString()
         {
            return(
               sprintf(
                  "{ id: [%d], year: [%d], title: [%s] }",
                  $this->mID,
                  $this->mYear,
                  $this->mTitle
               )
            );
         }
      }

      class MovieRating
      {
         var $mUser   = 0;
         var $mRating = 0;
         var $mDate   = "";

         function __construct($user, $rating, $date)
         {
            $this->mUser   = $user;
            $this->mRating = $rating;
            $this->mDate   = $date;
         }

         function toString()
         {
            return(
               sprintf(
                  "{ user: [%d], rating: [%d], date: [%s] }",
                  $this->mUser, $this->mRating, $this->mDate
               )
            );
         }
      }

      class All
      {
         var $mRating   = 0.0;
         var $mNRatings = 0;
         var $mYear     = "";
         var $mTitle    = "";

         function __construct($rating, $nRatings, $year,  $title)
         {
            $this->mRating   = $rating;
            $this->mNRatings = $nRatings;
            $this->mYear     = $year;
            $this->mTitle    = $title;
         }

         function toString()
         {
            return(
               sprintf(
                  "{ rating: [%s] , nratings: [%d], year: [%d], title: [%s] }",
                  $this->mRating,
                  $this->mNRatings,
                  $this->mYear,
                  $this->mTitle
               )
            );
         }
      }

      class Data
      {
         static function getMovieEntries($fileName, &$entries, &$errMsg)
         {
            $entries = array();

            if(($handle = @fopen($fileName,  "r")) === false)
            {
               $errMsg = "Could not open file: [$fileName]";
               return(false);
            }

            while(($line = fgets($handle)) !== false)
            {
               // We expect: movie-id, movie-year, movie-title
               // but some records have NULL for movie-year, we'll
               // skip those records

               if(!preg_match("/^(\\d+),(\\d+|NULL),(.*)$/", $line, $matches))
               {
                  fclose($handle);

                  $errMsg = sprintf(
                     "file: [%s], invalid line: [%s]",
                     $fileName, rtrim($line)
                  );

                  return(false);
               }

               $entries[] = new MovieEntry($matches[1], $matches[2], $matches[3]);
            }

            fclose($handle);

            return(true);
         }

         static function getMovieFileName($id)
         {
            return(
               DATA_TRAINING_SET_PATH . "/" .  sprintf("mv_%07d.txt", $id)
            );
         }

         static function idForTitle($title, &$id, &$errMsg)
         {
            if(!Data::getMovieEntries(DATA_MOVIE_TITLES_PATH, $entries, $errMsg))
               return(false);

            $regex = "/" . $title . "/i";

            foreach($entries as $entry)
            {
               if(preg_match($regex, $entry->mTitle))
               {
                  $id = $entry->mID;
                  return(true);
               }
            }

            $errMsg = "Not found";

            return(false);
         }

         static function getMovieRatings($id, &$ratings, &$errMsg)
         {
            $fileName = Data::getMovieFileName($id);

            if(($handle = @fopen($fileName, "r")) === false)
            {
               $errMsg = "Could not open file: [$fileName]";
               return(false);
            }

            $ratings = array();
            $first   = true;

            while(($line = fgets($handle)) !== false)
            {
               $line = trim($line);

               if($first)
               {
                  $first = false;

                  if(!preg_match("/^$id:$/", $line))
                  {
                     $errMsg = sprintf(
                        "file: [%s], invalid first line: [%s]", $fileName, $line
                     );

                     fclose($handle);

                     return(false);
                  }
               }
               else if(!preg_match("/^(\\d+),(\\d+),(\\d{4})-(\\d{2})-(\\d{2})$/",
                  $line, $match))
               {
                  $errMsg = sprintf(
                     "file: [%s], invalid line: [%s]", $fileName, $line
                  );

                  fclose($handle);

                  return(false);
               }
               else
               {
                  $ratings[] = new MovieRating(
                     $match[1],
                     $match[2],
                     sprintf("%d-%02d-%02d", $match[3], $match[4], $match[5])
                  );
               }
            }

            fclose($handle);

            return(true);
         }

         static function averageRatingForMovieEx($id, &$value, &$nRatings, &$errMsg)
         {
            if(!Data::getMovieRatings($id, $ratings, $errMsg))
               return(false);

            $sum = 0;

            foreach($ratings as $rating)
               $sum += $rating->mRating;

            $nRatings = count($ratings);

            $value = $sum / (double)$nRatings;

            return(true);
         }

         static function averageRatingForMovie($id, &$value, &$errMsg)
         {
            return(
               Data::averageRatingForMovieEx($id, $value, $nRatings, $errMsg)
            );
         }

         static function getAll($fileName, &$entries, &$errMsg)
         {
            if(($handle = @fopen($fileName, "r")) == false)
            {
               $errMsg = "Could not open: [$fileName]";
               return(false);
            }

            $entries = array();

            while(($line = fgets($handle)) !== false)
            {
               $line  = trim($line);

               if(!preg_match("/^([^,]+),([^,]+),([^,]+),(.*)$/", $line, $match))
               {
                  $errMsg = "Invalid line: [$line]";
                  return(false);
               }

               $entries[] = new All($match[1], $match[2], $match[3], $match[4]);
            }

            fclose($handle);

            return(true);
         }

         static function ratingsForUsers($users, &$userMap, &$errMsg)
         {
            $userMap = array();

            foreach($users as $user)
               $userMap[$user] = array();

            if(!Data::getMovieEntries(DATA_MOVIE_TITLES_PATH, $entries, $errMsg))
               return(false);

            foreach($entries as $index => $entry)
            {
               printf("($index) Scanning: %s\n", $entry->mTitle);

               if(!Data::getMovieRatings($entry->mID, $ratings, $errMsg))
                  return(false);

               foreach($ratings as $rating)
                  if(in_array($rating->mUser, $users))
                     $userMap[$rating->mUser][$entry->mID] = $rating->mRating;
            }

            return(true);
         }
      }
   }
?>
